﻿/* RGBLED_fade
 *
 * RGB LED vezérlése Hue, Saturation és Brightness (HSB/HSV ) paraméterekkel
 *  - A színezet (Hue) az A0 analóg bementre kötött potméterrel vezérelhető
 *  - A telítettség (Saturation) állandóan 255 (ez a maximális érték)
 *  - A fényességet (Brightness) a fadeVal változó adja meg (0..255)
 *
 * A getRGB() függvény elvi alapja itt található:
 * <http://www.codeproject.com/miscctrl/CPicker.asp>
 * dim_curve ötlet egy Jims becenevű felhasználótól származik:
 * Link: https://forum.arduino.cc/index.php/topic,40601.0.html
 * Szerző: Kasper Kamperman, 05-01-2010
 * Link: https://www.kasperkamperman.com/blog/arduino/arduino-programming-hsb-to-rgb/
 *
 * Kisebb módosítások, fordítás: 29-04-2020 Csarny István <hobbi@cspista.hu>
 */

/********************************************************************************
 * A dim_curve korrekciós táblázat az emberi látás nemlineáris voltát kompenzálja.
 * A getRGB() függvényben használjuk a fényesség és a telítettség korrekciójára
 * (az utóbbihoz invertálva), a fényerőszabályozás így természetesebbnek tűnik.
 * Az alábbi értékeket közelítőleg az y = round(pow( 2.0, [x+64]/40.0) - 1)
 * képlettel írhatjuk le, ahol x = 0 - 255 közötti érték, de vannak eltérések.
 ********************************************************************************/
const byte dim_curve[] = {
  0,   1,   1,   2,   2,   2,   2,   2,   2,   3,   3,   3,   3,   3,   3,   3,
  3,   3,   3,   3,   3,   3,   3,   4,   4,   4,   4,   4,   4,   4,   4,   4,
  4,   4,   4,   5,   5,   5,   5,   5,   5,   5,   5,   5,   5,   6,   6,   6,
  6,   6,   6,   6,   6,   7,   7,   7,   7,   7,   7,   7,   8,   8,   8,   8,
  8,   8,   9,   9,   9,   9,   9,   9,   10,  10,  10,  10,  10,  11,  11,  11,
  11,  11,  12,  12,  12,  12,  12,  13,  13,  13,  13,  14,  14,  14,  14,  15,
  15,  15,  16,  16,  16,  16,  17,  17,  17,  18,  18,  18,  19,  19,  19,  20,
  20,  20,  21,  21,  22,  22,  22,  23,  23,  24,  24,  25,  25,  25,  26,  26,
  27,  27,  28,  28,  29,  29,  30,  30,  31,  32,  32,  33,  33,  34,  35,  35,
  36,  36,  37,  38,  38,  39,  40,  40,  41,  42,  43,  43,  44,  45,  46,  47,
  48,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
  63,  64,  65,  66,  68,  69,  70,  71,  73,  74,  75,  76,  78,  79,  81,  82,
  83,  85,  86,  88,  90,  91,  93,  94,  96,  98,  99,  101, 103, 105, 107, 109,
  110, 112, 114, 116, 118, 121, 123, 125, 127, 129, 132, 134, 136, 139, 141, 144,
  146, 149, 151, 154, 157, 159, 162, 165, 168, 171, 174, 177, 180, 183, 186, 190,
  193, 196, 200, 203, 207, 211, 214, 218, 222, 226, 230, 234, 238, 242, 248, 255,
};

#define sensorPin   A0    // ide van kötve a potméter
#define ledPinR      3    // pwm kimenet a piros LED számára
#define ledPinG      5    // pwm kimenet a zöld LED számára
#define ledPinB      6    // pwm kimenet a kék LED számára
#define commonPin    4    // LED közös kivezetés
#define common_level HIGH // Közös anód: HIGH, katód: LOW

int sensorVal = 0;        // Az ADC-vel mért érték
int fadeVal   = 0;        // fényerősség, 0-255 között változik
int fadeSpeed = 4;        // az elhalványodás 'sebessége'

// A getRGB() függvény ebbe a tömbbe írja be az RGB értékeket
// amelyet a piros, zöl és kék LED-ek beállításához használunk
int rgb_colors[3];

int hue;                  // Színezet
int saturation;           // Telítettség
int brightness;           // Fényesség

void setup() {
  pinMode(commonPin, OUTPUT);             // 'Lusta' bekötéshez
  digitalWrite(commonPin, common_level);  // 'Lusta' bekötéshez
}

void loop()  {
  sensorVal = analogRead(sensorPin);     // Analóg beolvasás
  fadeVal = fadeVal + fadeSpeed;         // a fadeVal módosítása a fadeSpeed értékkel
  fadeVal = constrain(fadeVal, 0, 255);  // tartsa fadeVal értékét 0 és 255 között
  if (fadeVal == 255 || fadeVal == 0) {  // lefelé/fölfelé irányváltás
    fadeSpeed = -fadeSpeed;
  }
  hue = map(sensorVal, 0, 1023, 0, 359); // színezet 0 és 360 közötti szám legyen
  saturation = 255;                      // telítettség 255 legyen (ez a maximális érték)
  brightness = fadeVal;                  // fényesség 0 - 255 közötti érték

  getRGB(hue, saturation, brightness, rgb_colors);  // A HSB értékekből meghatározza az RGB értékeket

  if (common_level == HIGH) {            // Közös anód esete
    analogWrite(ledPinR, 255 - rgb_colors[0]);      // red value in index 0 of rgb_colors array
    analogWrite(ledPinG, 255 - rgb_colors[1]);      // green value in index 1 of rgb_colors array
    analogWrite(ledPinB, 255 - rgb_colors[2]);      // blue value in index 2 of rgb_colors array
  } else {                               // Közös katód esete
    analogWrite(ledPinR, rgb_colors[0]); // red value in index 0 of rgb_colors array
    analogWrite(ledPinG, rgb_colors[1]); // green value in index 1 of rgb_colors array
    analogWrite(ledPinB, rgb_colors[2]); // blue value in index 2 of rgb_colors array
  }
  delay(20);                             // A késleltetés szabályozza az elsötétítés sebességét
}

/* Átszámítja a hue, saturation és brightness ( HSB/HSV ) értékeket
   RGB adatokra. A fényesség és a telítettség korrekciójához
   a dim_curve táblázatot használjuk (a telítettséghez invertálva).
*/
void getRGB(int hue, int sat, int val, int colors[3]) {
  val = dim_curve[val];
  sat = 255 - dim_curve[255 - sat];

  int r;
  int g;
  int b;
  int base;

  if (sat == 0) { // Acromatic color (gray). Hue doesn't mind.
    colors[0] = val;
    colors[1] = val;
    colors[2] = val;
  } else  {

    base = ((255 - sat) * val) >> 8;

    switch (hue / 60) {
      case 0:
        r = val;
        g = (((val - base) * hue) / 60) + base;
        b = base;
        break;

      case 1:
        r = (((val - base) * (60 - (hue % 60))) / 60) + base;
        g = val;
        b = base;
        break;

      case 2:
        r = base;
        g = val;
        b = (((val - base) * (hue % 60)) / 60) + base;
        break;

      case 3:
        r = base;
        g = (((val - base) * (60 - (hue % 60))) / 60) + base;
        b = val;
        break;

      case 4:
        r = (((val - base) * (hue % 60)) / 60) + base;
        g = base;
        b = val;
        break;

      case 5:
        r = val;
        g = base;
        b = (((val - base) * (60 - (hue % 60))) / 60) + base;
        break;
    }

    colors[0] = r;
    colors[1] = g;
    colors[2] = b;
  }
}
